/*
  CatWeasel MK4 PCI Linux driver (c) 2002-2005 by Dirk Jagdmann <doj@cubic.org>

  This file is part of CatWeasel MK4 PCI Linux driver.

  The CatWeasel MK4 PCI Linux driver is free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  The CatWeasel MK4 PCI Linux driver is distributed in the hope that
  it will be useful, but WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the CatWeasel MK4 PCI Linux driver; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA 02111-1307 USA

  $Header: /code/cw/device/cw.h,v 1.15 2005/04/21 10:27:23 doj Exp $
*/

/*
  General header file used to include some standard linux stuff
*/

#ifndef CW__H
#define CW__H

#include <linux/config.h>

#if !defined(CONFIG_PCI)
#error Linux Kernel needs PCI support for CatWeasel MK4
#endif

#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#error Linux Version >= 2.6.0 needed for this driver
#endif

#include <linux/module.h>

#include <linux/init.h>

#include "cwsid.h"

/* generic registers */
#define CW_DATA_DIRECTION	0x02
#define CW_SELECT_BANK		0x03
#define CW_PORT_OUT_DIR		CW_SELECT_BANK
#define CW_PORT_IN_DIR		0x07

/* values for CW_SELECT_BANK */
#define CW_BANK_RESETFPGA	0x00
#define CW_BANK_FIFO		0x01
#define CW_BANK_FLOPPY		0x41
#define CW_BANK_IO		0x81

/* registers in FLOPPY bank */
#define CW_FLOPPY_JOYDAT	0xC0
#define CW_FLOPPY_JOYBUT	0xC8
#define CW_FLOPPY_JOYBUT_DIR	0xCC
#define CW_FLOPPY_KEYDAT	0xD0
#define CW_FLOPPY_KEYSTATUS	0xD4
#define CW_FLOPPY_SIDDAT	0xD8
#define CW_FLOPPY_SIDADDR	0xDC
#define CW_FLOPPY_MEMORY	0xE0
#define CW_FLOPPY_RESETPOINTER	0xE4
#define CW_FLOPPY_CONTROL	0xE8
#define CW_FLOPPY_OPTION	0xEC
#define CW_FLOPPY_START_A	0xF0
#define CW_FLOPPY_START_B	0xF4
#define CW_FLOPPY_IRQ		0xFC

/* registers in IO bank */
#define CW_IO_MOUSEY1		0xC0
#define CW_IO_MOUSEX1		0xC4
#define CW_IO_MOUSEY2		0xC8
#define CW_IO_MOUSEX2		0xCC
#define CW_IO_BUTTON		0xD0

extern int cw_iobase;
extern int sid_typ[];

int  cwsid_init(void);
void cwsid_exit(void);
unsigned char sid_read(unsigned char addr, int sid);

int  cwjoy_init(void);
void cwjoy_exit(void);

int  amikbd_init(void);
void amikbd_exit(void);
int  amikbd_poll1(void);
void amikbd_poll2(void);
void amikbd_poll3(void);

void cw_lock(unsigned char a);
void cw_unlock(void);

#endif
