/*
  CatWeasel MK4 PCI Linux driver (c) 2002-2004 by Dirk Jagdmann <doj@cubic.org>

  This file is part of CatWeasel MK4 PCI Linux driver.

  The CatWeasel MK4 PCI Linux driver is free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  The CatWeasel MK4 PCI Linux driver is distributed in the hope that
  it will be useful, but WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the CatWeasel MK4 PCI Linux driver; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA 02111-1307 USA

  $Header: /code/cw/device/cwjoy.c,v 1.19 2005/04/21 10:27:23 doj Exp $
*/

/*
  Code for accessing the joysticks, paddles and mice
*/

#include "cw.h"

#include <asm/io.h>
#include <linux/delay.h>
#include <linux/input.h>
#include <linux/timer.h>

static int cwjoy_rate=25;
MODULE_PARM(cwjoy_rate,"i");
MODULE_PARM_DESC(cwjoy_rate,"update joysticks and paddles x times per second (default=25)");

static int cw1351=0;
MODULE_PARM(cw1351, "i");
MODULE_PARM_DESC(cw1351, "use 1351 mouse. 1=port1, 2=port2, 3=both ports");

static int cwamiga=0;
MODULE_PARM(cwamiga, "i");
MODULE_PARM_DESC(cwamiga, "use Amiga mouse. 1=port1, 2=port2, 3=both ports");

static int cwatari=0;
MODULE_PARM(cwatari, "i");
MODULE_PARM_DESC(cwatari, "use Atari mouse. 1=port1, 2=port2, 3=both ports");

enum {
  Joystick1 = 0,
  Joystick2,
  Paddle1,
  Paddle2,
  Nummer,
};

#define Mouse1 Joystick1
#define Mouse2 Joystick2

static int paddle_but[] = { BTN_TRIGGER, BTN_THUMB, BTN_THUMB2, BTN_TOP, BTN_TOP2 };

static struct input_dev cwjoy_dev[Nummer];

static struct timer_list cwjoy_timer;

static int calcdelta1351(unsigned char n, unsigned char l)
{
  int d;
  if(l<16 && n>48)
    d=n-64+l;
  else if(l>48 && n<16)
    d=64-l+n;
  else
    d=n-l;
  return d;
}

static int calcdelta(unsigned char n, unsigned char l)
{
  int d;
  if(l<64 && n>192)
    d=n-256+l;
  else if(l>192 && n<64)
    d=256-l+n;
  else
    d=n-l;
  return d;
}

static void cwjoy_poll(unsigned long arg_)
{
  unsigned char d,b,x1,y1,x2,y2,bb,sid1_19,sid1_1a,sid2_19,sid2_1a;
  int fire1a,fire1b,fire1c,fire2a,fire2b,fire2c,key;

  /* reschedule ourself */
  mod_timer(&cwjoy_timer, jiffies+HZ/cwjoy_rate);

  /* read registers */
  cw_lock(CW_BANK_IO);
  y1=inb(cw_iobase + CW_IO_MOUSEY1);
  x1=inb(cw_iobase + CW_IO_MOUSEX1);
  y2=inb(cw_iobase + CW_IO_MOUSEY2);
  x2=inb(cw_iobase + CW_IO_MOUSEX2);
  bb=inb(cw_iobase + CW_IO_BUTTON);
  cw_unlock();

  cw_lock(CW_BANK_FLOPPY);

#if !USE_IRQ
  key=amikbd_poll1();
#endif

  /* prepare some status variables */
  d=inb(cw_iobase + CW_FLOPPY_JOYDAT);
  b=inb(cw_iobase + CW_FLOPPY_JOYBUT);

  fire1a=!(b&64);
  if(sid_typ[0]==SID_NONE)
    {
      /*check this*/
      fire1b=(bb&1)?1:0;
      fire1c=(bb&2)?1:0;
      sid1_19=sid1_1a=0;
    }
  else
    {
      sid1_19=sid_read(0x19, 0);
      sid1_1a=sid_read(0x1A, 0);
      fire1b=(sid1_19>128)?1:0;
      fire1c=(sid1_1a>128)?1:0;
    }

  fire2a=!(inb(cw_iobase + CW_PORT_IN_DIR)&0x10);
  if(sid_typ[1]==SID_NONE)
    {
      /*check this*/
      fire2b=(bb&4)?1:0;
      fire2c=(bb&8)?1:0;
      sid2_19=sid2_1a=0;
    }
  else
    {
      sid2_19=sid_read(0x19, 1);
      sid2_1a=sid_read(0x1A, 1);
      fire2b=(sid2_19>128)?1:0;
      fire2c=(sid2_1a>128)?1:0;
    }

#if !USE_IRQ
  if(key>=0)
    amikbd_poll2();
#endif

  cw_unlock();

#if !USE_IRQ
  if(key>=0)
    amikbd_poll3();
#endif

  /* send events to kernel */
  if((cwamiga|cwatari)&1)
    {
      static unsigned char lx=0, ly=0;
      input_report_rel(&cwjoy_dev[Mouse1], REL_X, calcdelta(x1, lx));
      input_report_rel(&cwjoy_dev[Mouse1], REL_Y, calcdelta(y1, ly));
      input_report_key(&cwjoy_dev[Mouse1], BTN_MOUSE, fire1a);
      input_report_key(&cwjoy_dev[Mouse1], BTN_RIGHT, fire1b);
      input_report_key(&cwjoy_dev[Mouse1], BTN_MIDDLE,fire1c);
      input_sync(&cwjoy_dev[Mouse1]);
      lx=x1, ly=y1;
    }
  else if(cw1351&1)
    {
      static unsigned char lx=0, ly=0;
      unsigned char nx=(sid1_19&127)>>1, ny=(sid1_1a&127)>>1;
      input_report_rel(&cwjoy_dev[Mouse1], REL_X,  calcdelta1351(nx, lx));
      input_report_rel(&cwjoy_dev[Mouse1], REL_Y, -calcdelta1351(ny, ly));
      input_report_key(&cwjoy_dev[Mouse1], BTN_MOUSE, fire1a);
      input_report_key(&cwjoy_dev[Mouse1], BTN_RIGHT, !(d&8));
      input_sync(&cwjoy_dev[Mouse1]);
      lx=nx;ly=ny;
    }
  else
    {
      if((int)(cwjoy_dev[Joystick1].private))
	{
	  input_report_abs(&cwjoy_dev[Joystick1], ABS_X, (d&1?0:1) - (d&2?0:1));
	  input_report_abs(&cwjoy_dev[Joystick1], ABS_Y, (d&4?0:1) - (d&8?0:1));
	  input_report_key(&cwjoy_dev[Joystick1], BTN_TRIGGER, fire1a);
	  input_sync(&cwjoy_dev[Joystick1]);
	}
      if((int)(cwjoy_dev[Paddle1].private))
	{
	  input_report_abs(&cwjoy_dev[Paddle1], ABS_X, sid1_19);
	  input_report_abs(&cwjoy_dev[Paddle1], ABS_Y, sid1_1a);
	  input_report_key(&cwjoy_dev[Paddle1], paddle_but[0], fire1a);
	  input_report_key(&cwjoy_dev[Paddle1], paddle_but[1], !(d&1));
	  input_report_key(&cwjoy_dev[Paddle1], paddle_but[2], !(d&2));
	  input_report_key(&cwjoy_dev[Paddle1], paddle_but[3], !(d&4));
	  input_report_key(&cwjoy_dev[Paddle1], paddle_but[4], !(d&8));
	  input_sync(&cwjoy_dev[Paddle1]);
	}

    }

  if((cwamiga|cwatari)&2)
    {
      static unsigned char lx=0, ly=0;
      input_report_rel(&cwjoy_dev[Mouse2], REL_X, calcdelta(x2, lx));
      input_report_rel(&cwjoy_dev[Mouse2], REL_Y, calcdelta(y2, ly));
      input_report_key(&cwjoy_dev[Mouse2], BTN_MOUSE, fire2a);
      input_report_key(&cwjoy_dev[Mouse2], BTN_RIGHT, fire2b);
      input_report_key(&cwjoy_dev[Mouse2], BTN_MIDDLE,fire2c);
      input_sync(&cwjoy_dev[Mouse2]);
      lx=x2, ly=y2;
    }
  else if(cw1351&2)
    {
      static unsigned char lx=0, ly=0;
      unsigned char nx=(sid2_19&127)>>1, ny=(sid2_1a&127)>>1;
      input_report_rel(&cwjoy_dev[Mouse2], REL_X,  calcdelta1351(nx, lx));
      input_report_rel(&cwjoy_dev[Mouse2], REL_Y, -calcdelta1351(ny, ly));
      input_report_key(&cwjoy_dev[Mouse2], BTN_MOUSE, fire2a);
      input_report_key(&cwjoy_dev[Mouse2], BTN_RIGHT, !(d&128));
      input_sync(&cwjoy_dev[Mouse2]);
      lx=nx;ly=ny;
    }
  else
    {
      if((int)(cwjoy_dev[Joystick2].private))
	{
	  input_report_abs(&cwjoy_dev[Joystick2], ABS_X, (d&16?0:1) - (d&32?0:1));
	  input_report_abs(&cwjoy_dev[Joystick2], ABS_Y, (d&64?0:1) - (d&128?0:1));
	  input_report_key(&cwjoy_dev[Joystick2], BTN_TRIGGER, fire2a);
	  input_sync(&cwjoy_dev[Joystick2]);
	}
      if((int)(cwjoy_dev[Paddle2].private))
	{
	  input_report_abs(&cwjoy_dev[Paddle2], ABS_X, sid2_19);
	  input_report_abs(&cwjoy_dev[Paddle2], ABS_Y, sid2_1a);
	  input_report_key(&cwjoy_dev[Paddle2], paddle_but[0], fire2a);
	  input_report_key(&cwjoy_dev[Paddle2], paddle_but[1], !(d&16));
	  input_report_key(&cwjoy_dev[Paddle2], paddle_but[2], !(d&32));
	  input_report_key(&cwjoy_dev[Paddle2], paddle_but[3], !(d&64));
	  input_report_key(&cwjoy_dev[Paddle2], paddle_but[4], !(d&128));
	  input_sync(&cwjoy_dev[Paddle2]);
	}
    }

}

static int open(struct input_dev *dev)
{
  ++(*((int*)&dev->private));
  return 0;
}

static void close(struct input_dev *dev)
{
  int *p=(int*)&(dev->private);
  if(--(*p) < 0)
    *p=0;
}

static int initialized=0;

int __init cwjoy_init(void)
{
  int j, usepaddle=0;
  static char str[Nummer][32];
  struct input_dev *d;

  /* check preconditions */
  if((cw1351&1) && sid_typ[0]==SID_NONE)
    {
      printk(KERN_ERR DEVICE_NAME " 1351 mouse on port 1 needs a SID in socket 1\n");
      return -1;
    }
  if((cw1351&2) && sid_typ[1]==SID_NONE)
    {
      printk(KERN_ERR DEVICE_NAME " 1351 mouse on port 2 needs a SID in socket 2\n");
      return -1;
    }

  cw_lock(CW_BANK_IO);
  /* configure amiga and atari mouse in CW */
  outb( ((~cwamiga)|cwatari)&3, cw_iobase + CW_IO_BUTTON);

  /* clear mice counters */
  outb(0, cw_iobase + CW_IO_MOUSEY1);
  outb(0, cw_iobase + CW_IO_MOUSEX1);
  outb(0, cw_iobase + CW_IO_MOUSEY2);
  outb(0, cw_iobase + CW_IO_MOUSEX2);

  cw_unlock();

  /* control port 1 */
  if((cw1351&1) || (cwamiga&1) || (cwatari&1))
    {
      /* 1351 Mouse 1 */
      d=&cwjoy_dev[Mouse1];
      init_input_dev(d);
      sprintf(str[Mouse1], "CatWeasel %i Mouse1", 1);
      d->name = str[Mouse1];
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      set_bit(BTN_MOUSE, d->keybit);
      set_bit(BTN_RIGHT, d->keybit);
      set_bit(BTN_MIDDLE, d->keybit);
      set_bit(EV_REL, d->evbit);
      set_bit(REL_X, d->relbit);
      set_bit(REL_Y, d->relbit);
      input_register_device(d);
    }
  else
    {
      /* Joystick 1 */
      d=&cwjoy_dev[Joystick1];
      init_input_dev(d);
      d->private=0;
      d->open=open;
      d->close=close;
      sprintf(str[Joystick1], "CatWeasel %i Joystick 1", 1);
      d->name = str[Joystick1];
      d->phys = str[Joystick1];
      d->uniq = "uniq Joystick 1";
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      set_bit(BTN_TRIGGER, d->keybit);
      set_bit(EV_ABS, d->evbit);
      set_bit(ABS_X, d->absbit);
      d->absmin[ABS_X] = -1; d->absmax[ABS_X] = +1;
      set_bit(ABS_Y, d->absbit);
      d->absmin[ABS_Y] = -1; d->absmax[ABS_Y] = +1;
      input_register_device(d);

      usepaddle+=1;
    }

  /* control port 2 */
  if((cw1351&2) || (cwamiga&2) || (cwatari&2))
    {
      /* 1351 Mouse 2 */
      d=&cwjoy_dev[Mouse2];
      init_input_dev(d);
      sprintf(str[Mouse2], "CatWeasel %i Mouse2", 1);
      d->name = str[Mouse2];
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      set_bit(BTN_MOUSE, d->keybit);
      set_bit(BTN_RIGHT, d->keybit);
      set_bit(BTN_MIDDLE, d->keybit);
      set_bit(EV_REL, d->evbit);
      set_bit(REL_X, d->relbit);
      set_bit(REL_Y, d->relbit);
      input_register_device(d);
    }
  else
    {
      /* Joystick 2 */
      d=&cwjoy_dev[Joystick2];
      init_input_dev(d);
      d->private=0;
      d->open=open;
      d->close=close;
      sprintf(str[Joystick2], "CatWeasel %i Joystick 2", 1);
      d->name = str[Joystick2];
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      set_bit(BTN_TRIGGER, d->keybit);
      set_bit(EV_ABS, d->evbit);
      set_bit(ABS_X, d->absbit);
      d->absmin[ABS_X] = -1; d->absmax[ABS_X] = 1;
      set_bit(ABS_Y, d->absbit);
      d->absmin[ABS_Y] = -1; d->absmax[ABS_Y] = 1;
      input_register_device(d);

      usepaddle+=2;
    }

  if(usepaddle&1)
    {
      /* Paddle 1*/
      d=&cwjoy_dev[Paddle1];
      init_input_dev(d);
      d->private=0;
      d->open=open;
      d->close=close;
      sprintf(str[Paddle1], "CatWeasel %i Paddle 1", 1);
      d->name = str[Paddle1];
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      for(j=0; j!=5; ++j)
	set_bit(paddle_but[j], d->keybit);
      set_bit(EV_ABS, d->evbit);
      set_bit(ABS_X, d->absbit);
      d->absmin[ABS_X] = 0; d->absmax[ABS_X] = 0xFF;
      set_bit(ABS_Y, d->absbit);
      d->absmin[ABS_Y] = 0; d->absmax[ABS_Y] = 0xFF;
      input_register_device(d);
    }
  if(usepaddle&2)
    {
      /* Paddle 2*/
      d=&cwjoy_dev[Paddle2];
      init_input_dev(d);
      d->private=0;
      d->open=open;
      d->close=close;
      sprintf(str[Paddle2], "CatWeasel %i Paddle 2", 1);
      d->name = str[Paddle2];
      d->id.bustype = BUS_GAMEPORT;
      d->id.vendor = 0xE159;
      d->id.product = 1;
      d->id.version = 0x0100;
      set_bit(EV_KEY, d->evbit);
      for(j=0; j!=5; ++j)
	set_bit(paddle_but[j], d->keybit);
      set_bit(EV_ABS, d->evbit);
      set_bit(ABS_X, d->absbit);
      d->absmin[ABS_X] = 0; d->absmax[ABS_X] = 0xFF;
      set_bit(ABS_Y, d->absbit);
      d->absmin[ABS_Y] = 0; d->absmax[ABS_Y] = 0xFF;
      input_register_device(d);
    }

  /* enable timer */
  if((cw1351 || cwamiga || cwatari) && cwjoy_rate<100)
    cwjoy_rate=100;

  init_timer(&cwjoy_timer);
  cwjoy_timer.function=cwjoy_poll;
  cwjoy_timer.expires=jiffies+HZ/cwjoy_rate;
  add_timer(&cwjoy_timer);

  initialized=1;
  return 0;
}

void __exit cwjoy_exit(void)
{
  if(initialized)
    {
      del_timer_sync(&cwjoy_timer);

      if((cw1351&1) || (cwamiga&1) || (cwatari&1))
	{
	  input_unregister_device(&cwjoy_dev[Mouse1]);
	}
      else
	{
	  input_unregister_device(&cwjoy_dev[Joystick1]);
	  input_unregister_device(&cwjoy_dev[Paddle1]);
	}
      if((cw1351&2) || (cwamiga&2) || (cwatari&2))
	{
	  input_unregister_device(&cwjoy_dev[Mouse2]);
	}
      else
	{
	  input_unregister_device(&cwjoy_dev[Joystick2]);
	  input_unregister_device(&cwjoy_dev[Paddle2]);
	}

      initialized=0;
    }
}
