/*
  CatWeasel MK4 PCI Linux driver (c) 2002-2004 by Dirk Jagdmann <doj@cubic.org>

  This file is part of CatWeasel MK4 PCI Linux driver.

  The CatWeasel MK4 PCI Linux driver is free software; you can
  redistribute it and/or modify it under the terms of the GNU General
  Public License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  The CatWeasel MK4 PCI Linux driver is distributed in the hope that
  it will be useful, but WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
  PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with the CatWeasel MK4 PCI Linux driver; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA 02111-1307 USA

  $Header: /code/cw/device/cwkey.c,v 1.14 2005/04/21 10:27:23 doj Exp $
*/

/*
  Code for accessing amiga keyboards. Based on Linux amiga keyboard
  code by Vojtech Pavlik and Hamish Macdonald.
*/

#include "cw.h"

#include <asm/io.h>
#include <linux/input.h>

#define FIX_FAKE_KEY_BUG 1
#define PRINTCODES 0

#if PRINTCODES
static
#include "keyname.c"
#endif

static unsigned char amikbd_keycode[0x78] = {
  [0]	= KEY_GRAVE,
  [1]	= KEY_1,
  [2]	= KEY_2,
  [3]	= KEY_3,
  [4]	= KEY_4,
  [5]	= KEY_5,
  [6]	= KEY_6,
  [7]	= KEY_7,
  [8]	= KEY_8,
  [9]	= KEY_9,
  [10]	= KEY_0,
  [11]	= KEY_MINUS,
  [12]	= KEY_EQUAL,
  [13]	= KEY_BACKSLASH,
  [15]	= KEY_KP0,
  [16]	= KEY_Q,
  [17]	= KEY_W,
  [18]	= KEY_E,
  [19]	= KEY_R,
  [20]	= KEY_T,
  [21]	= KEY_Y,
  [22]	= KEY_U,
  [23]	= KEY_I,
  [24]	= KEY_O,
  [25]	= KEY_P,
  [26]	= KEY_LEFTBRACE,
  [27]	= KEY_RIGHTBRACE,
  [29]	= KEY_KP1,
  [30]	= KEY_KP2,
  [31]	= KEY_KP3,
  [32]	= KEY_A,
  [33]	= KEY_S,
  [34]	= KEY_D,
  [35]	= KEY_F,
  [36]	= KEY_G,
  [37]	= KEY_H,
  [38]	= KEY_J,
  [39]	= KEY_K,
  [40]	= KEY_L,
  [41]	= KEY_SEMICOLON,
  [42]	= KEY_APOSTROPHE,
  [43]	= KEY_BACKSLASH,
  [45]	= KEY_KP4,
  [46]	= KEY_KP5,
  [47]	= KEY_KP6,
  [48]	= KEY_102ND,
  [49]	= KEY_Z,
  [50]	= KEY_X,
  [51]	= KEY_C,
  [52]	= KEY_V,
  [53]	= KEY_B,
  [54]	= KEY_N,
  [55]	= KEY_M,
  [56]	= KEY_COMMA,
  [57]	= KEY_DOT,
  [58]	= KEY_SLASH,
  [60]	= KEY_KPDOT,
  [61]	= KEY_KP7,
  [62]	= KEY_KP8,
  [63]	= KEY_KP9,
  [64]	= KEY_SPACE,
  [65]	= KEY_BACKSPACE,
  [66]	= KEY_TAB,
  [67]	= KEY_KPENTER,
  [68]	= KEY_ENTER,
  [69]	= KEY_ESC,
  [70]	= KEY_DELETE,
  [74]	= KEY_KPMINUS,
  [76]	= KEY_UP,
  [77]	= KEY_DOWN,
  [78]	= KEY_RIGHT,
  [79]	= KEY_LEFT,
  [80]	= KEY_F1,
  [81]	= KEY_F2,
  [82]	= KEY_F3,
  [83]	= KEY_F4,
  [84]	= KEY_F5,
  [85]	= KEY_F6,
  [86]	= KEY_F7,
  [87]	= KEY_F8,
  [88]	= KEY_F9,
  [89]	= KEY_F10,
  [90]	= KEY_KPLEFTPAREN,
  [91]	= KEY_KPRIGHTPAREN,
  [92]	= KEY_KPSLASH,
  [93]	= KEY_KPASTERISK,
  [94]	= KEY_KPPLUS,
  [95]	= KEY_HELP,
  [96]	= KEY_LEFTSHIFT,
  [97]	= KEY_RIGHTSHIFT,
  [98]	= KEY_CAPSLOCK,
  [99]	= KEY_LEFTCTRL,
  [100]	= KEY_LEFTALT,
  [101]	= KEY_RIGHTALT,
  [102]	= KEY_LEFTMETA,
  [103]	= KEY_RIGHTMETA
};

static const char *amikbd_messages[8] = {
  [0]	= KERN_ALERT	"amikbd: Ctrl-Amiga-Amiga reset warning\n",
  [1]	= KERN_WARNING	"amikbd: keyboard lost sync\n",
  [2]	= KERN_WARNING	"amikbd: keyboard buffer overflow\n",
  [3]	= KERN_WARNING	"amikbd: keyboard controller failure\n",
  [4]	= KERN_ERR	"amikbd: keyboard selftest failure\n",
  [5]	= KERN_INFO	"amikbd: initiate power-up key stream\n",
  [6]	= KERN_INFO	"amikbd: terminate power-up key stream\n",
  [7]	= KERN_WARNING	"amikbd: keyboard interrupt\n"
};

static struct input_dev amikbd_dev;

static char *amikbd_name = "Amiga keyboard";
static char *amikbd_phys = "CatWeasel Keyboard";

static unsigned char scancode;

int amikbd_poll1(void)
{
  /* check for new key */
  if(inb(cw_iobase + CW_FLOPPY_KEYSTATUS) & 0x80)
    return scancode = inb(cw_iobase + CW_FLOPPY_KEYDAT);
  return -1;
}

void amikbd_poll2(void)
{
  /* ack key read */
  outb(0, cw_iobase + CW_FLOPPY_KEYDAT);
}

void amikbd_poll3(void)
{
  const int down = !(scancode&128);
  scancode&=127;

  /* process key */
  if(scancode < sizeof(amikbd_keycode))
    {
      const unsigned char kcode=amikbd_keycode[scancode];

#if PRINTCODES
      printk(KERN_INFO "amikbd: %02x -> %02x %s\n", scancode, kcode, keyname(kcode));
#endif

      if (kcode == KEY_CAPSLOCK)
	{
#if 0
	  /* CapsLock is a toggle switch key on Amiga */
	  input_report_key(&amikbd_dev, kcode, 1);
	  input_report_key(&amikbd_dev, kcode, 0);
#endif
	}
#if FIX_FAKE_KEY_BUG
      else if(scancode == 0)
	{
	  /* upon init this keycode gets reported. We always generate a key up event to scancode 0 */
	  if(down)
	    {
	      input_report_key(&amikbd_dev, kcode, 1);
	      input_report_key(&amikbd_dev, kcode, 0);
	    }
	}
#endif
      else
	input_report_key(&amikbd_dev, kcode, down);

      input_sync(&amikbd_dev);
    }
  /* process error code */
  else
    printk(amikbd_messages[scancode - 0x78]);
}

static int initialized=0;

int __init amikbd_init(void)
{
  int i;

  init_input_dev(&amikbd_dev);
  set_bit(EV_KEY, amikbd_dev.evbit);
  set_bit(EV_REP, amikbd_dev.evbit);
  amikbd_dev.keycode = amikbd_keycode;
  amikbd_dev.keycodesize = sizeof(unsigned char);
  amikbd_dev.keycodemax = ARRAY_SIZE(amikbd_keycode);

  for (i = 0; i < sizeof(amikbd_keycode); i++)
    if (amikbd_keycode[i])
      set_bit(amikbd_keycode[i], amikbd_dev.keybit);

  amikbd_dev.name = amikbd_name;
  amikbd_dev.phys = amikbd_phys;
  amikbd_dev.id.bustype = BUS_AMIGA;
  amikbd_dev.id.vendor = 0x0001;
  amikbd_dev.id.product = 0x0001;
  amikbd_dev.id.version = 0x0100;

  input_register_device(&amikbd_dev);

  /* enable keyboard */
  cw_lock(CW_BANK_FLOPPY);
  outb(0, cw_iobase + CW_FLOPPY_KEYSTATUS);
  outb(0, cw_iobase + CW_FLOPPY_KEYDAT);
  cw_unlock();

  initialized=1;
  return 0;
}

void __exit amikbd_exit(void)
{
  if(initialized)
    {
      input_unregister_device(&amikbd_dev);
      initialized=0;
    }
}
