/*
  sid (c) 1999-2004 by Dirk Jagdmann <doj@cubic.org>

  This file is part of sid.

  sid is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  sid is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  License for more details.

  You should have received a copy of the GNU General Public License
  along with sid; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  
  $Header: /code/cw/sid/playlist.h,v 1.11 2004/05/14 16:01:24 doj Exp $
*/

#ifndef PLAYLIST__H
#define PLAYLIST__H

#include <string>
#include <vector>
#include <iostream>

#ifndef __NCURSES_H
#include <curses.h>
#undef clear
#endif

/**
   simple class for handling a playlist. It is soo bad, it's already scheduled for rewrite.
*/
class Playlist
{
 protected:

  struct item
  {
    std::string file;
    int song;
    item(const std::string& f="", int s=-1) : file(f), song(s) { }
    bool operator<(const item& b) const
    {
      if(file==b.file)
	return song<b.song;
      return file<b.file;
    }
    bool operator==(const item& b) const
    {
      return (file==b.file) || (song==b.song);
    }
    std::ostream& print(std::ostream& os) const { return os << file << " #" << song; }
  };

  /// container type for filenames
  typedef std::vector<item> array_t;

  /// filenames managed by this playlist
  array_t a;

  /// currently selected filename
  int current_;
  int cursor;

  WINDOW *playw;
  int width;
  int height;

  void addm3u(const std::string& fn);
  void addpls(const std::string& fn);
  void addplaylisthelper(std::string fn, const int song);

 public:
  Playlist();
  virtual ~Playlist();

  void init(int& xoffset, int& yoffset, int height_);

  void playinfo_print();

  void up();
  void down();
  void select() { current_=cursor; }

  size_t size() const { return a.size(); }

  const std::string current();
  const int currentsong();
  Playlist& next();
  Playlist& previous();
  Playlist& random();

  Playlist& add(const std::string&, const int song=-1);
  Playlist& randomize();
  Playlist& sort();

  friend std::ostream& operator<<(std::ostream& os, const Playlist::item& i);

 private:
  Playlist(const Playlist&);
  Playlist& operator=(const Playlist&);;
};

inline std::ostream& operator<<(std::ostream& os, const Playlist::item& i)
{
  return i.print(os);
}

#endif
